<?php

namespace FiloBlu\DisableShop\Controller\Index;

use Exception;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Area;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filesystem;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Framework\View\Result\Page;
use Magento\Newsletter\Model\Subscriber;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class ProductInfo
 * @package FiloBlu\DisableShop\Controller\Index
 */
class ProductInfo extends Action
{
    /**
     * @var JsonFactory
     */
    protected $_resultJsonFactory;
    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var TransportBuilder
     */
    protected $_transportBuilder;
    /**
     * @var Subscriber
     */
    protected $_subscriber;
    /**
     * @var Context
     */
    protected $context;

    /**
     * @return Redirect|Page
     */

    public function __construct(
        Context $context,
        Filesystem $fileSystem,
        TransportBuilder $transportBuilder,
        StoreManagerInterface $_storeManager,
        ScopeConfigInterface $scopeConfig,
        JsonFactory $resultJsonFactory,
        Subscriber $subscriber
    ) {
        parent::__construct($context);
        $this->fileSystem = $fileSystem;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $_storeManager;
        $this->_scopeConfig = $scopeConfig;
        $this->_resultJsonFactory = $resultJsonFactory;
        $this->_subscriber = $subscriber;
        $this->context = $context;
    }

    public function execute()
    {
        $resultJson = $this->_resultJsonFactory->create();
        $post = $this->getRequest()->getPostValue();

        $recipientEmail = $this->_scopeConfig->getValue(
            'filoblu_disableshop_section/ask_info/ask_info_email',
            ScopeInterface::SCOPE_STORE
        );

        $askInfoEmailTemplate = $this->_scopeConfig->getValue(
            'filoblu_disableshop_section/ask_info/ask_info_email_template',
            ScopeInterface::SCOPE_STORE
        );

        $askInfoEmailTemplateToCustomer = $this->_scopeConfig->getValue(
            'filoblu_disableshop_section/ask_info/ask_info_email_template_to_customer',
            ScopeInterface::SCOPE_STORE
        );

        $newsletterSub = $this->getRequest()->getParam('info_newsletter') ? true : false;

        // Email sender to store
        $fromEmailToStore = $post['info_email'];

        $templateVarsToStore = [
            'store' => $this->_storeManager->getStore(),
            'customer_email' => $post['info_email'],
            'product_name' => $post['product_name'],
            'product_sku' => $post['product_sku'],
        ];

        $fromToStore = ['email' => $fromEmailToStore, 'name' => ''];
        $toToStore = $recipientEmail;

        $templateOptions = [
            'area' => Area::AREA_FRONTEND,
            'store' => Store::DEFAULT_STORE_ID
        ];

        try {
            $transport = $this->_transportBuilder->setTemplateIdentifier($askInfoEmailTemplate)
                ->setTemplateOptions($templateOptions)
                ->setTemplateVars($templateVarsToStore)
                ->setFrom($fromToStore)
                ->addTo($toToStore)
                ->getTransport();
            $transport->sendMessage();
        } catch (LocalizedException $e) {
            return $resultJson->setData(['success' => false, 'error' => true, 'message' => __('There was a problem with the request: %1', $e->getMessage())]);
        } catch (Exception $e) {
            return $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
        }

        //Email sender to customer
        $templateVarsToCustomer = [
            'store' => $this->_storeManager->getStore(),
            'product_name' => $post['product_name']
        ];

        $fromEmailToCustomer = $recipientEmail;
        $fromToCustomer = ['email' => $fromEmailToCustomer, 'name' => ''];

        $toToCustomer = $post['info_email'];

        try {
            $transport = $this->_transportBuilder->setTemplateIdentifier($askInfoEmailTemplateToCustomer)
                ->setTemplateOptions($templateOptions)
                ->setTemplateVars($templateVarsToCustomer)
                ->setFrom($fromToCustomer)
                ->addTo($toToCustomer)
                ->getTransport();
            $transport->sendMessage();
        } catch (Exception $e) {
            return $resultJson->setData(['success' => false, 'error' => true, 'message' => __($e->getMessage())]);
        }

        try {
            if ($newsletterSub) {
                $isAlreadySubscribed = $this->isAlreadySubscribedToNewsletter($post['info_email']);
                if (!$isAlreadySubscribed) {
                    $this->_subscriber->subscribe($post['info_email']);
                    $this->context->getMessageManager()->addSuccessMessage(__('Thank you for your subscription.'));
                } else {
                    $this->context->getMessageManager()->addSuccessMessage(__('This email address is already subscribed.'));
                }
            }
        } catch (Exception $e) {
            $this->messageManager->addExceptionMessage($e, __('Unable to complete your newsletter subscription'));
        }

        $resultJson->setData(['success' => true, 'error' => false, 'message' => __('Info request recived correctly')]);

        return $resultJson;
    }

    /**
     * @param string $email
     * @return bool
     */
    private function isAlreadySubscribedToNewsletter($email)
    {
        $checkSubscriber = $this->_subscriber->loadByEmail($email);
        return $checkSubscriber->isSubscribed();
    }
}
