<?php

namespace FiloBlu\DisableShop\Helper;

use FiloBlu\DisableShop\Model\Logger\Logger;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Http\Context;
use Magento\Store\Model\ScopeInterface;

class Data extends AbstractHelper
{
    protected $_loggerDisableShop;
    protected $_httpContext;
    protected $_customerSession;

    /**
     * Data constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param Logger $logger
     * @param Context $httpContext
     * @param Session $customerSession
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        Logger $logger,
        Context $httpContext,
        Session $customerSession
    ) {
        $this->_loggerDisableShop = $logger;
        $this->_httpContext = $httpContext;
        $this->_customerSession = $customerSession;
        parent::__construct($context);
    }

    public function canDisableShop()
    {
        return $this->isCurrentUserExcluded($this->getExcludeCustomerGroupDisabledShop()) ? false : $this->getConfig("general/disable_shop");
    }

    private function isCurrentUserExcluded($customerGroupExcluded)
    {
        $isLogged = $this->_httpContext->getValue(\Magento\Customer\Model\Context::CONTEXT_AUTH);
        $_currentCustomerGroupId = '-1';
        if ($isLogged) {
            $_currentCustomerGroupId = '-2,';
            $_currentCustomerGroupId = $_currentCustomerGroupId . $this->_customerSession->getCustomer()->getGroupId();
        }
        $_arrayCurrentCustomerGroupId = explode(',', $_currentCustomerGroupId);
        $_arrayCustomerGroupExcluded = explode(',', $customerGroupExcluded);
        $response = (count(array_intersect($_arrayCurrentCustomerGroupId, $_arrayCustomerGroupExcluded)) != 0);
        return $response;
    }

    private function getExcludeCustomerGroupDisabledShop()
    {
        return $this->getConfig('general/disable_shop_exclude_customer_group');
    }

    public function getConfig($config_path, $section = 'filoblu_disableshop_section')
    {
        return $this->scopeConfig->getValue(
            $section . '/' . $config_path,
            ScopeInterface::SCOPE_STORE
        );
    }

    public function canHidePrice()
    {
        return $this->isCurrentUserExcluded($this->getExcludeCustomerGroupHidePrice()) ? false : $this->getConfig('general/hide_price');
    }

    private function getExcludeCustomerGroupHidePrice()
    {
        return $this->getConfig('general/hide_price_exclude_customer_group');
    }

    public function getCategoriesDisabledShop()
    {
        return $this->isCurrentUserExcluded($this->getExcludeCustomerGroupDisabledShop()) ? "" : $this->getConfig('general/disable_shop_category');
    }

    public function getCategoriesHidePrice()
    {
        return $this->isCurrentUserExcluded($this->getExcludeCustomerGroupHidePrice()) ? "" : $this->getConfig('general/hide_price_category');
    }

    public function log($message)
    {
        if ($this->getConfig('debug/enable_log')) {
            $this->_loggerDisableShop->info($message);
        }
    }
}
